<?php
/**
 * ###############################################
 *
 * SWIFT Framework
 * _______________________________________________
 *
 * @author		Varun Shoor
 *
 * @package		SWIFT
 * @copyright	Copyright (c) 2001-2012, Kayako
 * @license		http://www.kayako.com/license
 * @link		http://www.kayako.com
 *
 * ###############################################
 */

/**
 * The Main Installer
 *
 * @author Varun Shoor
 */
class SWIFT_SetupDatabase_livechat extends SWIFT_SetupDatabase
{
	// Core Constants
	const PAGE_COUNT = 1;

	/**
	 * Constructor
	 *
	 * @author Varun Shoor
	 * @return bool "true" on Success, "false" otherwise
	 */
	public function __construct()
	{
		parent::__construct(APP_LIVECHAT);

		return true;
	}

	/**
	 * Destructor
	 *
	 * @author Varun Shoor
	 * @return bool "true" on Success, "false" otherwise
	 */
	public function __destruct()
	{
		parent::__destruct();

		return true;
	}

	/**
	 * Loads the table into the container
	 *
	 * @author Varun Shoor
	 * @return bool "true" on Success, "false" otherwise
	 */
	public function LoadTables()
	{
		// ======= VISITORPULLS =======
		$this->AddTable('visitorpulls', new SWIFT_SetupDatabaseTable(TABLE_PREFIX . "visitorpulls", "visitorsessionid C(32) PRIMARY NOTNULL,
																staffid I DEFAULT '0' NOTNULL,
																dateline I DEFAULT '0' NOTNULL"));
		$this->AddIndex('visitorpulls', new SWIFT_SetupDatabaseIndex("visitorpulls1", TABLE_PREFIX . "visitorpulls", "staffid"));
		// ======= VISITORPULLS =======
		$this->AddTable('visitorpulls2', new SWIFT_SetupDatabaseTable(TABLE_PREFIX . "visitorpulls2", "visitorsessionid C(32) PRIMARY NOTNULL,
																staffid I DEFAULT '0' NOTNULL,
																dateline I DEFAULT '0' NOTNULL"));
		$this->AddIndex('visitorpulls2', new SWIFT_SetupDatabaseIndex("visitorpulls21", TABLE_PREFIX . "visitorpulls2", "staffid"));

		// ======= CHATDATA =======
		$this->AddTable('chatdata', new SWIFT_SetupDatabaseTable(TABLE_PREFIX . "chatdata", "chatdataid I PRIMARY AUTO NOTNULL,
																chatobjectid I DEFAULT '0' NOTNULL,
																contents X2"));
		$this->AddIndex('chatdata', new SWIFT_SetupDatabaseIndex("cobjid", TABLE_PREFIX . "chatdata", "chatobjectid"));

		// ======= MESSAGEQUEUE =======
		$this->AddTable('messagequeue', new SWIFT_SetupDatabaseTable(TABLE_PREFIX . "messagequeue", "messagequeueid I PRIMARY AUTO NOTNULL,
																chatobjectid I DEFAULT '0' NOTNULL,
																chatchildid C(32) DEFAULT '' NOTNULL,
																staffid I DEFAULT '0' NOTNULL,
																dateline I DEFAULT '0' NOTNULL,
																name C(150) DEFAULT '' NOTNULL,
																contents X2,
																msgtype C(15) DEFAULT '' NOTNULL,
																guid C(150) DEFAULT '' NOTNULL,
																submittype I2 DEFAULT '0' NOTNULL"));
		$this->AddIndex('messagequeue', new SWIFT_SetupDatabaseIndex("messagequeue1", TABLE_PREFIX . "messagequeue", "chatobjectid, chatchildid"));
		$this->AddIndex('messagequeue', new SWIFT_SetupDatabaseIndex("messagequeue2", TABLE_PREFIX . "messagequeue", "dateline"));
		$this->AddIndex('messagequeue', new SWIFT_SetupDatabaseIndex("messagequeue3", TABLE_PREFIX . "messagequeue", "chatobjectid, staffid"));
		$this->AddIndex('messagequeue', new SWIFT_SetupDatabaseIndex("messagequeue4", TABLE_PREFIX . "messagequeue", "guid"));

		// ======= MESSAGEDATA =======
		$this->AddTable('messagedata', new SWIFT_SetupDatabaseTable(TABLE_PREFIX . "messagedata", "messagedataid I PRIMARY AUTO NOTNULL,
																messageid I DEFAULT '0' NOTNULL,
																contenttype I2 DEFAULT '0' NOTNULL,
																contents X2"));
		$this->AddIndex('messagedata', new SWIFT_SetupDatabaseIndex("messagedata1", TABLE_PREFIX . "messagedata", "messageid, contenttype"));
		$this->AddIndex('messagedata', new SWIFT_SetupDatabaseIndex("messagedata2", TABLE_PREFIX . "messagedata", "contenttype, messageid"));

		// ======= VISITORNOTEDATA =======
		$this->AddTable('visitornotedata', new SWIFT_SetupDatabaseTable(TABLE_PREFIX . "visitornotedata", "visitornotedataid I PRIMARY AUTO NOTNULL,
																visitornoteid I DEFAULT '0' NOTNULL,
																contents X2"));
		$this->AddIndex('visitornotedata', new SWIFT_SetupDatabaseIndex("visitornotedata", TABLE_PREFIX . "visitornotedata", "visitornoteid"));

		// ======= CANNEDRESPONSEDATA =======
		$this->AddTable('cannedresponsedata', new SWIFT_SetupDatabaseTable(TABLE_PREFIX . "cannedresponsedata", "cannedresponsedataid I PRIMARY AUTO NOTNULL,
																cannedresponseid I DEFAULT '0' NOTNULL,
																contents X2"));
		$this->AddIndex('cannedresponsedata', new SWIFT_SetupDatabaseIndex("cannedresponsedata1", TABLE_PREFIX . "cannedresponsedata", "cannedresponseid"));

		// ======= NEW IN v4.00.00 =======
		$this->AddTable('chatskilllinks', new SWIFT_SetupDatabaseTable(TABLE_PREFIX . "chatskilllinks", "chatskilllinkid I PRIMARY AUTO NOTNULL,
																chatskillid I DEFAULT '0' NOTNULL,
																staffid I DEFAULT '0' NOTNULL"));
		$this->AddIndex('chatskilllinks', new SWIFT_SetupDatabaseIndex("chatskilllinks1", TABLE_PREFIX . "chatskilllinks", "chatskillid"));
		$this->AddIndex('chatskilllinks', new SWIFT_SetupDatabaseIndex("chatskilllinks2", TABLE_PREFIX . "chatskilllinks", "staffid"));

		// ======= VISITORRULECRITERIA =======
		$this->AddTable('visitorrulecriteria', new SWIFT_SetupDatabaseTable(TABLE_PREFIX . "visitorrulecriteria", "visitorrulecriteriaid I PRIMARY AUTO NOTNULL,
																visitorruleid I DEFAULT '0' NOTNULL,
																name C(100) DEFAULT '' NOTNULL,
																ruleop I2 DEFAULT '0' NOTNULL,
																rulematch C(255) DEFAULT '' NOTNULL"));
		$this->AddIndex('visitorrulecriteria', new SWIFT_SetupDatabaseIndex("visitorrulecriteria1", TABLE_PREFIX . "visitorrulecriteria", "visitorruleid"));

		// ======= VISITORRULEACTIONS =======
		$this->AddTable('visitorruleactions', new SWIFT_SetupDatabaseTable(TABLE_PREFIX . "visitorruleactions", "visitorruleactionid I PRIMARY AUTO NOTNULL,
																visitorruleid I DEFAULT '0' NOTNULL,
																actiontype C(100) DEFAULT '' NOTNULL,
																actionname C(100) DEFAULT '' NOTNULL,
																actionvalue C(255) DEFAULT '' NOTNULL"));
		$this->AddIndex('visitorruleactions', new SWIFT_SetupDatabaseIndex("visitorruleactions1", TABLE_PREFIX . "visitorruleactions", "visitorruleid"));

		return true;
	}

	/**
	 * Get the Page Count for Execution
	 *
	 * @author Varun Shoor
	 * @return bool "true" on Success, "false" otherwise
	 */
	public function GetPageCount()
	{
		return self::PAGE_COUNT;
	}

	/**
	 * Function that does the heavy execution
	 *
	 * @author Varun Shoor
	 * @param int $_pageIndex The Page Index
	 * @return bool "true" on Success, "false" otherwise
	 */
	public function Install($_pageIndex)
	{
		parent::Install($_pageIndex);

		if (strtolower(DB_TYPE) == "mysql")
		{
			$this->Query(new SWIFT_SetupDatabaseSQL("ALTER TABLE ". TABLE_PREFIX ."visitorfootprints TYPE = HEAP"));
			$this->Query(new SWIFT_SetupDatabaseSQL("ALTER TABLE ". TABLE_PREFIX ."visitorpulls TYPE = HEAP"));
			$this->Query(new SWIFT_SetupDatabaseSQL("ALTER TABLE ". TABLE_PREFIX ."chatchilds TYPE = HEAP"));
		}

		// ======= DEPARTMENTS =======
		$_DepartmentObject = SWIFT_Department::Insert($this->Language->Get('coregeneral'), APP_LIVECHAT, SWIFT_PUBLIC, 0, 0, false, array());
		$this->Database->AutoExecute(TABLE_PREFIX . 'templategroups', array('departmentid_livechat' => $_DepartmentObject->GetDepartmentID()),
				'UPDATE', "1 = 1");

		$_staffIDList = $_staffGroupIDList = array();
		$this->Database->Query("SELECT * FROM " . TABLE_PREFIX . "staff");
		while ($this->Database->NextRecord())
		{
			$_staffIDList[] = $this->Database->Record['staffid'];
		}

		$this->Database->Query("SELECT * FROM " . TABLE_PREFIX . "staffgroup");
		while ($this->Database->NextRecord())
		{
			$_staffGroupIDList[] = $this->Database->Record['staffgroupid'];
		}

		SWIFT_StaffAssign::AssignDepartmentList($_DepartmentObject, $_staffIDList);
		SWIFT_StaffGroupAssign::AssignDepartmentList($_DepartmentObject, $_staffGroupIDList);

		// ======= CRON =======
		SWIFT_Cron::Create('livechat', 'LiveChat', 'LiveChatMinute', 'Indexing', '0', '10', '0', true);

		/**
		 * ---------------------------------------------
		 * CUSTOM DATA
		 * ---------------------------------------------
		 */
		$_SWIFT_VisitorGroupObject = SWIFT_VisitorGroup::Insert($this->Language->Get('lc_searchenginevisitors'), '#d8f1ff');

		$_chatSkillID = SWIFT_ChatSkill::Insert($this->Language->Get('lc_skillsales'), '', $_staffIDList);
		$_chatSkillID = SWIFT_ChatSkill::Insert($this->Language->Get('lc_skillsupport'), '', $_staffIDList);
		$_chatSkillID = SWIFT_ChatSkill::Insert($this->Language->Get('lc_skillaccounts'), '', $_staffIDList);

		$this->ExecuteQueue();

		return true;
	}

	/**
	 * Uninstalls the App
	 *
	 * @author Varun Shoor
	 * @return bool "true" on Success, "false" otherwise
	 */
	public function Uninstall()
	{
		parent::Uninstall();

		SWIFT_Cron::DeleteOnName(array('livechat'));

		SWIFT_Department::DeleteOnApp(array(APP_LIVECHAT));

		SWIFT_Rating::DeleteOnType(array(SWIFT_Rating::TYPE_CHATHISTORY, SWIFT_Rating::TYPE_CHATSURVEY));

		return true;
	}

	/**
	 * Upgrade the App
	 *
	 * @author Varun Shoor
	 * @param bool $_isForced
	 * @return bool "true" on Success, "false" otherwise
	 */
	public function Upgrade($_isForced = false)
	{
		return parent::Upgrade($_isForced);
	}
}
?>