<?php
/**
 * ###############################################
 *
 * SWIFT Framework
 * _______________________________________________
 *
 * @author		Varun Shoor
 *
 * @package		SWIFT
 * @copyright	Copyright (c) 2001-2012, Kayako
 * @license		http://www.kayako.com/license
 * @link		http://www.kayako.com
 *
 * ###############################################
 */

/**
 * The Main Installer
 *
 * @author Varun Shoor
 */
class SWIFT_SetupDatabase_base extends SWIFT_SetupDatabase
{
	// Core Constants
	const PAGE_COUNT = 1;

	/**
	 * Constructor
	 *
	 * @author Varun Shoor
	 * @return bool "true" on Success, "false" otherwise
	 */
	public function __construct()
	{
		parent::__construct(APP_BASE);

		return true;
	}

	/**
	 * Destructor
	 *
	 * @author Varun Shoor
	 * @return bool "true" on Success, "false" otherwise
	 */
	public function __destruct()
	{
		parent::__destruct();

		return true;
	}

	/**
	 * Loads the table into the container
	 *
	 * @author Varun Shoor
	 * @return bool "true" on Success, "false" otherwise
	 */
	public function LoadTables()
	{
		// ======= SIGNATURES =======
		$this->AddTable('signatures', new SWIFT_SetupDatabaseTable(TABLE_PREFIX . "signatures", "signatureid I PRIMARY AUTO NOTNULL,
																dateline I DEFAULT '0' NOTNULL,
																staffid I DEFAULT '0' NOTNULL,
																signature X"));
		$this->AddIndex('signatures', new SWIFT_SetupDatabaseIndex("signatures1", TABLE_PREFIX . "signatures", "staffid"));

		// ======= TEMPLATEDATA =======
		$this->AddTable('templatedata', new SWIFT_SetupDatabaseTable(TABLE_PREFIX . "templatedata", "templatedataid I PRIMARY AUTO NOTNULL,
																templateid I DEFAULT '0' NOTNULL,
																contents XL,
																contentsdefault XL"));
		$this->AddIndex('templatedata', new SWIFT_SetupDatabaseIndex("templatedata1", TABLE_PREFIX . "templatedata", "templateid"));

		// ======= COMMENTDATA =======
		$this->AddTable('commentdata', new SWIFT_SetupDatabaseTable(TABLE_PREFIX . "commentdata", "commentdataid I PRIMARY AUTO NOTNULL,
																commentid I DEFAULT '0' NOTNULL,
																contents X2"));
		$this->AddIndex('commentdata', new SWIFT_SetupDatabaseIndex("commentdata1", TABLE_PREFIX . "commentdata", "commentid"));


		// ======= CUSTOMFIELDOPTIONLINKS =======
		$this->AddTable('customfieldoptionlinks', new SWIFT_SetupDatabaseTable(TABLE_PREFIX . "customfieldoptionlinks", "customfieldoptionlinkid I PRIMARY AUTO NOTNULL,
																customfieldid I DEFAULT '0' NOTNULL,
																customfieldoptionid I DEFAULT '0' NOTNULL"));
		$this->AddIndex('customfieldoptionlinks', new SWIFT_SetupDatabaseIndex("customfieldoptionlinks1", TABLE_PREFIX . "customfieldoptionlinks", "customfieldid"));
		$this->AddIndex('customfieldoptionlinks', new SWIFT_SetupDatabaseIndex("customfieldoptionlinks2", TABLE_PREFIX . "customfieldoptionlinks", "customfieldoptionid"));

		// ======= USERNOTEDATA =======
		$this->AddTable('usernotedata', new SWIFT_SetupDatabaseTable(TABLE_PREFIX . "usernotedata", "usernotedataid I PRIMARY AUTO NOTNULL,
																usernoteid I DEFAULT '0' NOTNULL,
																notecontents XL"));
		$this->AddIndex('usernotedata', new SWIFT_SetupDatabaseIndex("usernotedata1", TABLE_PREFIX . "usernotedata", "usernoteid"));

		for ($_index = 1; $_index <= 10; $_index++) {
			// ======= GEOIPISP =======
			$this->AddTable('geoipisp' . $_index, new SWIFT_SetupDatabaseTable(TABLE_PREFIX . "geoipisp" . $_index, "ipfrom I8 DEFAULT '0' NOTNULL,
																	ipto I8 DEFAULT '0' NOTNULL,
																	isp C(255) DEFAULT '' NOTNULL"));
			$this->AddIndex('geoipisp' . $_index, new SWIFT_SetupDatabaseIndex("geoipisp1", TABLE_PREFIX . "geoipisp" . $_index, "ipto"));

			// ======= GEOIPORGANIZATION =======
			$this->AddTable('geoiporganization' . $_index, new SWIFT_SetupDatabaseTable(TABLE_PREFIX . "geoiporganization" . $_index, "ipfrom I8 DEFAULT '0' NOTNULL,
																	ipto I8 DEFAULT '0' NOTNULL,
																	organization C(255) DEFAULT '' NOTNULL"));
			$this->AddIndex('geoiporganization' . $_index, new SWIFT_SetupDatabaseIndex("geoiporganization1", TABLE_PREFIX . "geoiporganization" . $_index, "ipto"));

			// ======= GEOIPCITYBLOCKS =======
			$this->AddTable('geoipcityblocks' . $_index, new SWIFT_SetupDatabaseTable(TABLE_PREFIX . "geoipcityblocks" . $_index, "ipfrom I8 DEFAULT '0' NOTNULL,
																	ipto I8 DEFAULT '0' NOTNULL,
																	blockid I DEFAULT '0' NOTNULL"));
			$this->AddIndex('geoipcityblocks' . $_index, new SWIFT_SetupDatabaseIndex("geoipcityblocks1", TABLE_PREFIX . "geoipcityblocks" . $_index, "ipto"));
			$this->AddIndex('geoipcityblocks' . $_index, new SWIFT_SetupDatabaseIndex("geoipcityblocks2", TABLE_PREFIX . "geoipcityblocks" . $_index, "blockid"));

			// ======= GEOIPNETSPEED =======
			$this->AddTable('geoipnetspeed' . $_index, new SWIFT_SetupDatabaseTable(TABLE_PREFIX . "geoipnetspeed" . $_index, "ipfrom I8 DEFAULT '0' NOTNULL,
																	ipto I8 DEFAULT '0' NOTNULL,
																	netspeed C(100) DEFAULT '' NOTNULL"));
			$this->AddIndex('geoipnetspeed' . $_index, new SWIFT_SetupDatabaseIndex("geoipnetspeed1", TABLE_PREFIX . "geoipnetspeed" . $_index, "ipto"));
		}

		// ======= GEOIPCITIES =======
		$this->AddTable('geoipcities', new SWIFT_SetupDatabaseTable(TABLE_PREFIX . "geoipcities", "blockid I PRIMARY DEFAULT '0' NOTNULL,
																country C(5) DEFAULT '' NOTNULL,
																region C(100) DEFAULT '' NOTNULL,
																city C(255) DEFAULT '' NOTNULL,
																postalcode C(100) DEFAULT '' NOTNULL,
																latitude C(100) DEFAULT '' NOTNULL,
																longitude C(100) DEFAULT '' NOTNULL,
																metrocode C(100) DEFAULT '' NOTNULL,
																areacode C(100) DEFAULT '' NOTNULL"));

		// ======= ATTACHMENTCHUNKS =======
		$this->AddTable('attachmentchunks', new SWIFT_SetupDatabaseTable(TABLE_PREFIX ."attachmentchunks", "chunkid I PRIMARY AUTO NOTNULL,
																		attachmentid I DEFAULT '0' NOTNULL,
																		contents X2,
																		notbase64 I2 DEFAULT '0' NOTNULL"));
		$this->AddIndex('attachmentchunks', new SWIFT_SetupDatabaseIndex("attachmentchunks1", TABLE_PREFIX ."attachmentchunks", "attachmentid"));

		// ======= SEARCHINDEX ======
		$this->AddTable('searchindex', new SWIFT_SetupDatabaseTable(TABLE_PREFIX . "searchindex", "objid INT NOTNULL,
														subobjid INT DEFAULT '0',
														type SMALLINT,
														ft TEXT"), self::TABLETYPE_MYISAM); // FULLTEXT index (created in Install())
		$this->AddIndex('searchindex', new SWIFT_SetupDatabaseIndex("searchindex1", TABLE_PREFIX . "searchindex", "objid"));	   // Searching by id
		$this->AddIndex('searchindex', new SWIFT_SetupDatabaseIndex("searchindex2", TABLE_PREFIX . "searchindex", "type, objid"));	   // Searching by type
		$this->AddIndex('searchindex', new SWIFT_SetupDatabaseIndex("searchindex3", TABLE_PREFIX . "searchindex", "objid, type")); // Searching for object by type
		$this->AddIndex('searchindex', new SWIFT_SetupDatabaseIndex("searchindex4", TABLE_PREFIX . "searchindex", "objid, subobjid, type")); // Searching for object + sub-object by type

		/**
		 * ---------------------------------------------
		 * NOTIFICATIONS
		 * ---------------------------------------------
		 */

		// ======= NOTIFICATIONCRITERIA =======
		$this->AddTable('notificationcriteria', new SWIFT_SetupDatabaseTable(TABLE_PREFIX . "notificationcriteria", "notificationcriteriaid I PRIMARY AUTO NOTNULL,
																notificationruleid I DEFAULT '0' NOTNULL,
																name C(200) DEFAULT '' NOTNULL,
																ruleop I2 DEFAULT '0' NOTNULL,
																rulematch C(255) DEFAULT '' NOTNULL,
																rulematchtype I2 DEFAULT '0' NOTNULL"));
		$this->AddIndex('notificationcriteria', new SWIFT_SetupDatabaseIndex("notificationcriteria1", TABLE_PREFIX . "notificationcriteria", "notificationruleid"));

		// ======= NOTIFICATIONACTIONS =======
		$this->AddTable('notificationactions', new SWIFT_SetupDatabaseTable(TABLE_PREFIX . "notificationactions", "notificationactionid I PRIMARY AUTO NOTNULL,
																notificationruleid I DEFAULT '0' NOTNULL,
																actiontype I2 DEFAULT '0' NOTNULL,
																contents C(255) DEFAULT '' NOTNULL"));
		$this->AddIndex('notificationactions', new SWIFT_SetupDatabaseIndex("notificationactions1", TABLE_PREFIX . "notificationactions", "notificationruleid"));

		return true;

	}





	/**
	 * Get the Page Count for Execution
	 *
	 * @author Varun Shoor
	 * @return bool "true" on Success, "false" otherwise
	 */
	public function GetPageCount()
	{
		return self::PAGE_COUNT;
	}

	/**
	 * Function that does the heavy execution
	 *
	 * @author Varun Shoor
	 * @param int $_pageIndex The Page Index
	 * @return bool "true" on Success, "false" otherwise
	 */
	public function Install($_pageIndex)
	{
		parent::Install($_pageIndex);

		$_SWIFT_UserGroup_guest = SWIFT_UserGroup::Create($this->Language->Get('coreguest'), SWIFT_UserGroup::TYPE_GUEST, true);
		$_SWIFT_UserGroup_registered = SWIFT_UserGroup::Create($this->Language->Get('coreregistered'), SWIFT_UserGroup::TYPE_REGISTERED, true);
		$_userGroupID_guest = $_SWIFT_UserGroup_guest->GetUserGroupID();
		$_userGroupID_registered = $_SWIFT_UserGroup_registered->GetUserGroupID();

		$this->Insert(new SWIFT_SetupDatabaseInsertSQL(TABLE_PREFIX."templategroups", array('languageid' => '1', 'guestusergroupid' => intval($_userGroupID_guest), 'regusergroupid' => intval($_userGroupID_registered), 'title' => 'Default', 'description' => $this->Language->Get('coredeftgroup'), 'companyname' => $_POST["companyname"], 'ismaster' => '1', 'grouppassword' => '', 'restrictgroups' => '0', 'isdefault' => '1', 'loginshareid' => 0, 'loginapi_appid' => 0, 'ticketstatusid' => '1', 'priorityid' => '1', 'tickettypeid' => '1', 'tickets_promptpriority' => '1', 'departmentid' => '1', 'isenabled' => '1')));

		$this->ExecuteQueue();

		if (strtolower(DB_TYPE) == "mysql")
		{
			$this->Query(new SWIFT_SetupDatabaseSQL("ALTER TABLE " . TABLE_PREFIX . "attachmentchunks CHANGE contents contents MEDIUMBLOB DEFAULT '' NOT NULL;"));
			$this->Query(new SWIFT_SetupDatabaseSQL("CREATE FULLTEXT INDEX fulltextsearch on " . TABLE_PREFIX. "searchindex (ft);"));
		}

		// ======= STAFFGROPUPS =======
		$_StaffGroupObject_Admin = SWIFT_StaffGroup::Insert($this->Language->Get('coreadministrator'), true);
		$_StaffGroupObject_Staff = SWIFT_StaffGroup::Insert($this->Language->Get('corestaff'), false);

		// ======= STAFF =======
		$_SWIFT_StaffObject = SWIFT_Staff::Create($_POST['firstname'], $_POST['lastname'], '', $_POST['username'], $_POST['password'], $_StaffGroupObject_Admin->GetStaffGroupID(), $_POST['email'], '', '', false, true, '');

		// ======= USER & ORGANIZATION =======
		$_SWIFT_UserOrganizationObject = SWIFT_UserOrganization::Create($_POST['companyname'], SWIFT_UserOrganization::TYPE_RESTRICTED, array());
		SWIFT_User::Create($_userGroupID_registered, $_SWIFT_UserOrganizationObject->GetUserOrganizationID(), SWIFT_User::SALUTATION_NONE,
				$_POST['firstname'] . ' ' . $_POST['lastname'], '', '', true, SWIFT_User::ROLE_MANAGER, array($_POST['email']),
				$_POST['password'], false, false, false, false,	false, false, false, true);

		// ======= WIDGET =======
		SWIFT_Widget::Create('PHRASE:widgethome', 'home', APP_BASE, '/Core/Default/Index', '', '{$themepath}icon_widget_home_small.png', 0, true, false, true, true, SWIFT_Widget::VISIBLE_ALL, false);
		SWIFT_Widget::Create('PHRASE:widgetregister', 'register', APP_BASE, '/Base/UserRegistration/Register', '{$themepath}icon_widget_register.gif', '{$themepath}icon_widget_register_small.png', 1, false, true, true, true, SWIFT_Widget::VISIBLE_GUESTS, false);

		// ======= CRON =======
		SWIFT_Cron::Create('cronhourlycleanup', 'Base', 'BaseHourly', 'Cleanup', '-1', '0', '0', true);
		SWIFT_Cron::Create('crondailycleanup', 'Base', 'BaseDaily', 'Cleanup', '0', '0', '-1', true);

		// ======= NOTIFICATION RULE =======
		$_notificationCriteria = array(array('userevent', '1', 'newuser', '1'));
		$_notificationActions = array(array(SWIFT_NotificationAction::ACTION_EMAILSTAFFCUSTOM, $_SWIFT_StaffObject->GetStaffID()), array(SWIFT_NotificationAction::ACTION_POOLCUSTOM, $_SWIFT_StaffObject->GetStaffID(), '0'));
		SWIFT_NotificationRule::Create($this->Language->Get('snewuser'), SWIFT_NotificationRule::TYPE_USER, 1, $_notificationCriteria, $_notificationActions, $_SWIFT_StaffObject->GetStaffID());

		return true;
	}

	/**
	 * Uninstalls the app
	 *
	 * @author Varun Shoor
	 * @return bool "true" on Success, "false" otherwise
	 */
	public function Uninstall()
	{
		parent::Uninstall();

		return true;
	}

	/**
	 * Upgrade the App
	 *
	 * @author Varun Shoor
	 * @param bool $_isForced
	 * @return bool "true" on Success, "false" otherwise
	 */
	public function Upgrade($_isForced = false)
	{
		return parent::Upgrade($_isForced);
	}

	/**
	 * Upgrade from 4.01.191
	 *
	 * @author Varun Shoor
	 * @return bool "true" on Success, "false" otherwise
	 * @throws SWIFT_Exception If the Class is not Loaded
	 */
	public function Upgrade_4_01_191() {
		if (!$this->GetIsClassLoaded()) {
			throw new SWIFT_Exception(SWIFT_CLASSNOTLOADED);

			return false;
		}

		// Cleanup old templatedata entries: http://dev.kayako.com/browse/SWIFT-1132
		$this->Database->Query("DELETE FROM " . TABLE_PREFIX . "templatedata WHERE templateid <> ALL (SELECT templateid from " . TABLE_PREFIX . "templates)");

		return true;
	}

	/**
	 * Upgrade from 4.01.176
	 *
	 * @author Jamie Edwards
	 * @return bool "true" on Success, "false" otherwise
	 * @throws SWIFT_Exception If the Class is not Loaded
	 */
	public function Upgrade_4_01_176() {
		if (!$this->GetIsClassLoaded()) {
			throw new SWIFT_Exception(SWIFT_CLASSNOTLOADED);
			return false;
		}

		$this->Database->Query("ALTER TABLE " . TABLE_PREFIX . "searchindex ADD FULLTEXT (ft)");

		return true;
	}

	/**
	 * Upgrade from 4.01.326
	 *
	 * @author Parminder Singh
	 * @return bool "true" on Success, "false" otherwise
	 * @throws SWIFT_Exception If the Class is not Loaded
	 */
	public function Upgrade_4_01_326() {
		if (!$this->GetIsClassLoaded()) {
			throw new SWIFT_Exception(SWIFT_CLASSNOTLOADED);
			return false;
		}

		// SWIFT-1636: All outgoing mail is X-priority 1 after 4.01.325 upgrade
		$_newValue = 3;	// Normal
		if ($this->Settings->Get('cpu_maildefaultpriority') == 'High') {
			$_newValue = 1;
		} else if ($this->Settings->Get('cpu_maildefaultpriority') == 'Low') {
			$_newValue = 5;
		}

		$this->Settings->UpdateKey('settings', 'cpu_maildefaultpriority', $_newValue);

		return true;
	}

	/**
	 * Upgrade from 4.01.342
	 * Fixes the non existant signatures for staff
	 *
	 * @author Varun Shoor
	 * @return bool "true" on Success, "false" otherwise
	 * @throws SWIFT_Exception If the Class is not Loaded
	 */
	public function Upgrade_4_01_342() {
		if (!$this->GetIsClassLoaded()) {
			throw new SWIFT_Exception(SWIFT_CLASSNOTLOADED);
			return false;
		}

		$_staffContainer = $_staffIDList = array();

		$this->Database->Query("SELECT * FROM " . TABLE_PREFIX . "staff");
		while ($this->Database->NextRecord()) {
			$_staffContainer[$this->Database->Record['staffid']] = $this->Database->Record;

			$_staffIDList[] = $this->Database->Record['staffid'];
		}

		$_signatureStaffIDList = array();
		$this->Database->Query("SELECT staffid FROM " . TABLE_PREFIX . "signatures");
		while ($this->Database->NextRecord()) {
			$_signatureStaffIDList[] = $this->Database->Record['staffid'];
		}

		foreach ($_staffContainer as $_staffID => $_staff) {
			if (in_array($_staffID, $_signatureStaffIDList)) {
				continue;
			}

			$this->Database->AutoExecute(TABLE_PREFIX . 'signatures', array('dateline' => DATENOW, 'staffid' => intval($_staffID), 'signature' => ''), 'INSERT');
		}

		SWIFT_Staff::RebuildCache();

		return true;
	}

	/**
	 * Upgrade from 4.40.1148
	 * Move Ticket rating setting to global rating setting
	 *
	 * @author Parminder Singh
	 * @return bool "true" on Success, "false" otherwise
	 * @throws SWIFT_Exception If the Class is not Loaded
	 */
	public function Upgrade_4_40_1149() {

		if (!$this->GetIsClassLoaded()) {
			throw new SWIFT_Exception(SWIFT_CLASSNOTLOADED);
			return false;
		}

		$_staffGroupSettingsContainer = array();
		$this->Database->Query("SELECT staffgroupid, value FROM " . TABLE_PREFIX . "staffgroupsettings WHERE name LIKE 'staff_tcanratings'");
		while ($this->Database->NextRecord()) {
			$_staffGroupSettingsContainer[$this->Database->Record['staffgroupid']] = $this->Database->Record;
		}

		if (!count($_staffGroupSettingsContainer)) {
			return true;
		}

		$_staffGroupIDList = array();
		$this->Database->Query("SELECT staffgroupid FROM " . TABLE_PREFIX . "staffgroup");
		while ($this->Database->NextRecord()) {
			$_staffGroupIDList[] = $this->Database->Record['staffgroupid'];
		}

		foreach ($_staffGroupSettingsContainer as $_staffGroupID => $_staffGroupSettings) {
			if (!in_array($_staffGroupID, $_staffGroupIDList)) {
				continue;
			}

			$this->Database->AutoExecute(TABLE_PREFIX . 'staffgroupsettings', array('staffgroupid' => $_staffGroupSettings['staffgroupid'], 'name' => 'staff_canviewratings', 'value' => $_staffGroupSettings['value']), 'INSERT');
			$this->Database->AutoExecute(TABLE_PREFIX . 'staffgroupsettings', array('staffgroupid' => $_staffGroupSettings['staffgroupid'], 'name' => 'staff_canupdateratings', 'value' => $_staffGroupSettings['value']), 'INSERT');
		}

		// Now delete the entry
		$this->Database->Query("DELETE FROM " . TABLE_PREFIX . "staffgroupsettings WHERE name LIKE 'staff_tcanratings'");

		SWIFT_StaffGroup::RebuildCache();

		return true;
	}

}
?>